local fs = require "nixio.fs"

m = Map("iec104-slave", translate("IEC-60870 Slave"))

section_global = m:section(TypedSection, "global", translate("Enable/ Disable"))
section_global.anonymous = false
section_global.addremove = false
section_global.name = "global"

option_enabled = section_global:option(Flag, "enabled", translate("Enabled"),
    translate("Enable IEC-60870 Slave"))
option_enabled.rmempty = false
option_enabled.default = "1"

s = m:section(TypedSection, "iec104_slave", "IEC-60870 Slave Configuration")
s.anonymous = true


-- iec104 slave configuration --

no_of_masters = s:option(Value, "no_of_connections", "Number Of Master Connections", translate("Maximum number of simultaneous IEC104-Master connections"))
no_of_masters.datatype = "integer"
no_of_masters.default = "1"

cyclic_time_interval = s:option(Value, "cyclic_time_interval", "Cyclic Time Interval", translate("Set time interval to send (Unsolicited) ASDU with COT as a Cyclic"))
cyclic_time_interval.datatype = "integer"


k = s:option(Value, "k", "K", translate("Maximum unacknowledged transmitted APDUs"))
k.datatype = "integer"
k.default = "12"

w = s:option(Value, "w", "W", translate("Maximum unacknowledged received APDUs"))
w.datatype = "integer"
w.default = "8"

t0 = s:option(Value, "t0", "T0", translate("Amount of time that the IEC60870 Master will attempt to connect to the device"))
t0.datatype = "range(1,30)"
t0.default = "2"

t1 = s:option(Value, "t1", "T1", translate("Time to wait for a acknowledgment (“ACK”) to a transmitted APDU"))
t1.datatype = "integer"
t1.default = "15"

t2 = s:option(Value, "t2", "T2", translate("Time to wait before sending a supervisory APDU ACK"))
t2.datatype = "integer"
t2.default = "10"

t3 = s:option(Value, "t3", "T3", translate("Idle time before sending TEST APDU"))
t3.datatype = "integer"
t3.default = "20"

sizeof_cot = s:option(ListValue, "sizeof_cot", "Size Of COT", translate("Specify the number of octets in an ASDU COT field"))
sizeof_cot:value("1", translate("1"))
sizeof_cot:value("2", translate("2"))
sizeof_cot.default = "2"

oa = s:option(Value, "oa", "OA", translate("This is required when there is more than one controlled station(master/client)"))
oa.datatype = "range(0,254)"
oa.default = "0"

ca = s:option(Value, "ca", "CA", translate("Common Address or ASDU Address"))
ca.datatype = "range(0,254)"
ca.default = "1"

sizeof_ca = s:option(ListValue, "sizeof_ca", "Size Of CA", translate("Size Of Common Address or ASDU Address for IEC101 is 1 and for  IEC104 is 2"))
sizeof_ca:value("1", translate("1"))
sizeof_ca:value("2", translate("2"))
sizeof_ca.default = "2"

sizeof_ioa = s:option(ListValue, "sizeof_ioa", "Size Of IOA", translate("Size Of Information Object Address"))
sizeof_ioa:value("1", translate("1"))
sizeof_ioa:value("2", translate("2"))
sizeof_ioa:value("3", translate("3"))
sizeof_ioa.default = "3"

max_asdu_length = s:option(ListValue, "max_asdu_length", "Max. ASDU length", translate("Maximum size of ASDU that is generated,Its value is 249 for IEC104 and 254 for IEC101."))
max_asdu_length:value("249", translate("249"))
max_asdu_length:value("254", translate("254"))
max_asdu_length.default = "249"



-- iec104 slave configuration --

s2 = m:section(TypedSection, "iec104_ioas", translate("IEC-60870 IOAs Configuration"))
s2.anonymous = true
s2.template = "cbi/tblsection"

register_type = s2:option(DummyValue, "register_type", "Register Type")

ioa = s2:option(Value, "ioa", "IOA")
ioa.datatype = "integer"

ioa_type = s2:option(ListValue, "type_id", "IOA Type")
ioa_type:value("30", translate(" Single Point Information"))
ioa_type:value("31", translate(" Double Point Information"))
ioa_type:value("34", translate(" Measured Value- normalized value"))
ioa_type:value("35", translate(" Measured Value- scaled value"))
ioa_type:value("36", translate(" Measured Value- short floating point value"))


send_spontaneous_on_high = s2:option(Flag, "send_spontaneous_on_high", "Send Spontaneous on High", translate("Only for Single and Double Point Info."))

send_spontaneous_on_low = s2:option(Flag, "send_spontaneous_on_low", "Send Spontaneous on Low", translate("Only for Single and Double Point Info."))

send_spontaneous_on_indeterminate = s2:option(Flag, "send_spontaneous_on_indeterminate", "Send Spontaneous on Indeterminate", translate("Only for Double Point Info."))

spontaneous_ioa_high = s2:option(Value, "spontaneous_ioa_high", "Spontaneous IOA High", translate("Only for Single and Double Point Info."))
spontaneous_ioa_high.datatype = "integer"

spontaneous_ioa_low = s2:option(Value, "spontaneous_ioa_low", "Spontaneous IOA Low", translate("Only for Single and Double Point Info."))
spontaneous_ioa_low.datatype = "integer"

spontaneous_ioa_indeterminate = s2:option(Value, "spontaneous_ioa_indeterminate", "Spontaneous IOA Indeterminate", translate("Only for Double Point Info."))
spontaneous_ioa_indeterminate.datatype = "integer"

spontaneous_type_id = s2:option(ListValue, "spontaneous_type_id", "Spontaneous IOA Type")
spontaneous_type_id:value("30", translate(" Single Point Information"))
spontaneous_type_id:value("31", translate(" Double Point Information"))
spontaneous_type_id.default = "30"

scaling_a = s2:option(Value, "scaling_a", translate("Scaling A"), translate(""))
scaling_a.datatype = "float"
scaling_a.default = 1

scaling_b = s2:option(Value, "scaling_b", translate("Scaling B"), translate(""))
scaling_b.datatype = "float"
scaling_b.default = 0

hi_threashold = s2:option(Value, "high_threshold", translate("High Threshold"), translate(""))
hi_threashold.datatype = "range(0,100000)"

low_threashold = s2:option(Value, "low_threshold", translate("Low Threshold"), translate(""))
low_threashold.datatype = "range(0,100000)"

scaling_a_ioa = s2:option(Value, "scaling_a_ioa", translate("Scaling A IOA"), translate(""))
scaling_a_ioa.datatype = "integer"

scaling_b_ioa = s2:option(Value, "scaling_b_ioa", translate("Scaling B IOA"), translate(""))
scaling_b_ioa.datatype = "integer"

hi_threashold_ioa = s2:option(Value, "high_threshold_ioa", translate("High Threshold IOA"), translate(""))
hi_threashold_ioa.datatype = "integer"

low_threashold_ioa = s2:option(Value, "low_threshold_ioa", translate("Low Threshold IOA"), translate(""))
low_threashold_ioa.datatype = "integer"

scaling_type_id = s2:option(ListValue, "scaling_type_id", "Scaling IOA Type")
scaling_type_id:value("34", translate(" Measured Value- normalized value"))
scaling_type_id:value("35", translate(" Measured Value- scaled value"))
scaling_type_id:value("36", translate(" Measured Value- short floating point value"))

threshold_type_id = s2:option(ListValue, "threshold_type_id", "Threshold IOA Type")
threshold_type_id:value("34", translate(" Measured Value- normalized value"))
threshold_type_id:value("35", translate(" Measured Value- scaled value"))
threshold_type_id:value("36", translate(" Measured Value- short floating point value"))


return m
